/**
  **************************************************************************
  * @file     main.c
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "mc_lib.h"

/** @addtogroup AT32F415_pmsm_foc_demo
  * @{
  */
crm_clocks_freq_type crm_clocks_freq_struct = {0};

void disable_swo_debug_config(void);
void check_configured_core_clock(void);

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  system_clock_config();

  disable_swo_debug_config();

  systick_init();

  mc_delay_init();

  button_exint_init();

  led_config();

  nvic_config();

  adc_ordinary_config();

  adc_preempt_config();

  tmr_pwm_init();

  speed_timer_init();

  encoder_timer_init();
#if defined MT_METHOD
  encoder_capture_timer_init();
#endif

  mode_switch_init();

  /* get system clock */
  crm_clocks_freq_get(&crm_clocks_freq_struct);

  /* uart initialization */
  uart_init(&ui_usart);
  dma_uart_configuration();
  uart_rx_init();

  /* enable pwm timer */
  enable_pwm_timer();

  /*delay for hardware stable */
  mc_delay_ms(500);

  /* current offset initialization */
  I_offset_init();

  charge_boot_cap();

  firmware_id = get_fw_id();
  param_initial_rdy = mc_param_init(firmware_id);

  param_init();

  led_blink();

  while(1)
  {
    if (ctrl_mode_cmd != ctrl_mode_old)
    {
      param_clear();
      start_stop_btn_flag = RESET;
      if (ctrl_mode_cmd == POSITION_CTRL)
      {
        angle.cmd_final = pos.val * PULSE_TO_ANGLE;
        pos.cmd_new = pos.val;
        pos.cmd_final = pos.cmd_new;
        pos.command = pos.cmd_final;
      }
      ctrl_mode = ctrl_mode_cmd;
      ctrl_mode_old = ctrl_mode;
    }

    if (esc_state != esc_state_old)
    {
      switch(esc_state)
      {
      case ESC_STATE_IDLE:
        foc_rdy = RESET;
        pwm_switch_off();
        param_clear();
        led_off(ERROR_LED_PORT, ERROR_LED_GPIO_PIN);
        break;

      case ESC_STATE_SAFETY_READY:
        foc_rdy = RESET;
        pwm_switch_off();
        param_clear();
        break;

      case ESC_STATE_ANGLE_INIT:
        break;

      case ESC_STATE_STARTING:
        break;

      case ESC_STATE_ENC_ALIGN:
        if (ctrl_mode != OPEN_LOOP_CTRL)
        {
          if (encoder.align != SUCCEED)
          {
            encoder.align = PROCESSING;
          }
          foc_rdy = SET;
          pwm_switch_on();
        }
        break;

      case ESC_STATE_RUNNING:
        foc_rdy = SET;
        pwm_switch_on();
        break;

      case ESC_STATE_FREE_RUN:
        foc_rdy = RESET;
        pwm_switch_off();
        param_clear();
        break;

      case ESC_STATE_BRAKING:
        foc_rdy = RESET;
        pwm_switch_off();
        param_clear();
        start_stop_btn_flag = RESET;
        break;

      case ESC_STATE_ERROR:
        foc_rdy = RESET;
        pwm_switch_off();
        param_clear();
        start_stop_btn_flag = RESET;
        led_on(ERROR_LED_PORT, ERROR_LED_GPIO_PIN);
        break;

      case ESC_STATE_I_TUNE:
        param_clear();
        foc_rdy = SET;
        pwm_switch_on();
        break;

      case ESC_STATE_AUTO_LEARN:
        break;
#ifdef MOTOR_PARAM_IDENTIFY
      case ESC_STATE_WINDING_PARAM_ID:
        motor_parameter_ID_config();
        break;
#endif
      case ESC_STATE_NONE:
        break;
      }
      esc_state_old = esc_state;
    }
  }
}

/**
  * @brief  disable swo debug configuration
  * @param  none
  * @retval none
  */
void disable_swo_debug_config(void)
{
  crm_periph_clock_enable(CRM_IOMUX_PERIPH_CLOCK, TRUE);
  gpio_pin_remap_config(SWJTAG_GMUX_010, TRUE);

  DEBUGMCU->ctrl_bit.trace_ioen = FALSE;
//  DEBUGMCU->ctrl_bit.trace_ioen = TRUE;
}

/**
  * @brief  Check the core frequency matching the required frequency
  * @param  none
  * @retval none
  */
void check_configured_core_clock(void)
{
  if (system_core_clock != SYSTEM_CORE_CLOCK)
  {
    printf("Core freq.: %d, does not match the required freq. : %d", system_core_clock, SYSTEM_CORE_CLOCK);
    while(1);
  }
}

/**
  * @}
  */
